#-----------------------------------------------------------------------
# Gráfico de controle para a fração de itens não conformes.

# Número de itens não conformes em amostras de tamanho n = 50.
x0 <- c(12, 15, 8, 10, 4, 7, 16, 9, 14, 10, 5, 6, 17, 12, 22, 8, 10, 5,
        13, 11, 20, 18, 24, 15, 9, 12, 7, 13, 9, 6)
i0 <- seq_along(x0)

# Proporção amostral de itens não conformes
n <- 50
p0 <- x0/n

x <- x0
p <- p0
i <- i0

plot(p ~ i, type = "o")

# Linhas do gráfico de controle.
L <- list(LC = mean(p))
L <- within(L, {
    m <- 3 * sqrt(LC * (1 - LC)/n)
    LIC <- LC - m
    LSC <- LC + m
    rm(m)
})

ylim <- extendrange(c(p, unlist(L)), f = 0.1)
plot(p ~ i, type = "o", ylim = ylim)
with(L, abline(h = c(LC, LIC, LSC), lty = c(1, 2, 2)))

# r <- identify(x = i, y = p, n = 2)
r <- c(15, 23)

L <- list(LC = mean(p[-r]))
L <- within(L, {
    m <- 3 * sqrt(LC * (1 - LC)/n)
    LIC <- LC - m
    LSC <- LC + m
    rm(m)
})

ylim <- extendrange(c(p, unlist(L)), f = 0.1)
plot(p ~ i, type = "o", ylim = ylim)
with(L, abline(h = c(LC, LIC, LSC), lty = c(1, 2, 2)))
text(x = i[r], y = p[r], labels = c("Material", "Operador"), pos = 3)

x1 <- c(9, 6, 12, 5, 6, 4, 6, 3, 7, 6, 2, 4, 3, 6, 5, 4, 8, 5, 6, 7, 5,
        6, 3, 5)
p1 <- x1/n

M <- list(LC = mean(p1))
M <- within(M, {
    m <- 3 * sqrt(LC * (1 - LC)/n)
    LIC <- LC - m
    LSC <- LC + m
    rm(m)
})

x <- c(x0, x1)
i <- seq_along(x)
p <- c(p0, p1)

ylim <- extendrange(c(p, unlist(L), unlist(M)), f = 0.1)
plot(p ~ i, type = "o", ylim = ylim)
with(L, abline(h = c(LC, LIC, LSC), lty = c(1, 2, 2)))
with(M, abline(h = c(LC, LIC, LSC), lty = c(1, 2, 2), col = 2))
abline(v = length(x0), lty = 3)

x2 <- c(8, 7, 5, 6, 4, 5, 2, 3, 4, 7, 6, 5, 5, 3, 7, 9, 6, 10, 4, 3, 5,
        8, 11, 9, 7, 3, 5, 2, 1, 4, 5, 3, 7, 6, 4, 4, 6, 8, 5, 6)
p2 <- x2/n

x <- c(x0, x1, x2)
i <- seq_along(x)
p <- c(p0, p1, p2)

ylim <- extendrange(c(p, unlist(L), unlist(M)), f = 0.1)
plot(p ~ i, type = "o", ylim = ylim)
with(L, abline(h = c(LC, LIC, LSC), lty = c(1, 2, 2)))
with(M, abline(h = c(LC, LIC, LSC), lty = c(1, 2, 2), col = 2))
abline(v = length(x0), lty = 3)

#-----------------------------------------------------------------------

library(qcc)

qcc(data = x0, type = "p", sizes = n)
qcc(data = x1, type = "p", sizes = n, newdata = x2, newsizes = n)

#-----------------------------------------------------------------------
# Tamanho de amostra baseada na aproximação Poisson.

# Deseja-se que a probabilidade de pelo menos um item defeituoso seja no
# mínimo de gamma. Então Pr(X >= 1) >= gamma. Aproximando a binomial com
# a Poisson fazendo lambda = n * p.

# Probabilidade mínima de detectar pelo menos 1 item defeituoso.
gamma <- 0.95

# Probabilidade de defeito em um item (teórico).
p <- 0.01

# Grid de valores para lambda.
lambda <- seq(0.05, 5, by = 0.05)

# Como obter a probabilidade desejada com a ppois().
ppois(1, lambda = 1)                     # Pr(X <= 1).
1 - ppois(1, lambda = 1)                 # Pr(X > 1) = Pr(X >= 2).
ppois(0, lambda = 1)                     # Pr(X <= 0).
dpois(0, lambda = 1)                     # Pr(X = 0).
1 - ppois(0, lambda = 1)                 # Pr(X > 0).
ppois(0, lambda = 1, lower.tail = FALSE) # Pr(X > 0) = Pr(X >= 1).

# P(X >= 1 | lambda = i).
px <- 1 - dpois(0, lambda = lambda)
px <- ppois(0, lambda = lambda, lower.tail = FALSE)

plot(px ~ lambda, type = "l")
abline(h = gamma)

# Cria uma função por aproximação de segmentos de reta.
lamb <- approxfun(x = px, y = lambda)

# Na realidade, tem-se expressão analítica para Pr(D >= 1) ~ \lambda.
curve(1 - exp(-lambda), from = 0, to = 5, xname = "lambda")
abline(h = gamma)

# E o calculo é analítico.
-log(1 - gamma)

# Como lambda = n * p, então n é o tamanho de amostra mínimo necessário.
n <- ceiling(-log(1 - gamma)/p)

plot(px ~ lambda, type = "l")
curve(1 - exp(-lambda),
      from = 0, to = 5, xname = "lambda",
      add = TRUE, lty = 2, col = 4, lwd = 2)
abline(h = gamma, v = n * p, col = 2)

#-----------------------------------------------------------------------
# Curva característica de Operação.

# Limites de controle do processo.
M
n <- 50
p

# Pr(D < n * LSC) - Pr(D <= n * LIC)
lim <- pmax(floor(n * c(M$LSC, M$LIC)), 0)
lim

pgrid <- seq(0.001, 0.999, by = 0.001)
length(pgrid)

beta <- pbinom(lim[1], size = n, prob = pgrid) -
    pbinom(lim[2], size = n, prob = pgrid)

plot(beta ~ pgrid, type = "l",
     ylab = expression(beta), xlab = "p")
abline(h = 1 - 2 * pnorm(-3), lty = 2, col = 2)
abline(v = M, lty = 2, col = 2)

# Comprimento médio de sequência.
# Prob. de mostrar fora de controle quando está em controle.
cms0 <- 1/(2 * pnorm(-3))
cms0

# Se o processo agora tem p = 35.
bt <- pbinom(lim[1], size = n, prob = 0.15) -
    pbinom(lim[2], size = n, prob = 0.15)
cm1 <- 1/(1 - bt)
cm1

#-----------------------------------------------------------------------
# Maneiras de calcular os limites de controle.

M <- L
M$LC * n

# Aproximação Normal.
M

# Simulação.
s <- replicate(10000, {
    rbinom(n = 1, size = n, prob = M$LC)/n
})

# Reamostragem.
r <- replicate(10000, {
    sample(x0, replace = TRUE)/n
})

quantile(r, probs = pnorm(c(-3, 3)))
quantile(s, probs = pnorm(c(-3, 3)))

# O tuque está no weights.
m <- length(x0)
n <- 50
m0 <- glm(cbind(x0, 50 - x0) ~ 1,
          family = "binomial",
          weights = rep(1/m, m))

plot(profile(m0))
abline(h = c(-3, 3), lty = 2, col = 2)

# Intervalo de perfil de verossimilhança.
binomial()$linkinv(confint(m0, level = 1 - pnorm(-3)))

#-----------------------------------------------------------------------
